@@ -253,9 +253,12 @@ namespace opkele {
 			util::tidy_doc_t td = util::tidy_doc_t::create();
 			if(!td)
 			    throw exception_tidy(OPKELE_CP_ "failed to create htmltidy document");
-#ifndef NDEBUG
-			td.opt_set(TidyQuiet,false);
+#ifdef NDEBUG
+			td.opt_set(TidyQuiet,true);
 			td.opt_set(TidyShowWarnings,false);
+#else /* NDEBUG */
+			td.opt_set(TidyQuiet,false);
+			td.opt_set(TidyShowWarnings,true);
 #endif /* NDEBUG */
 			td.opt_set(TidyForceOutput,true);
 			td.opt_set(TidyXhtmlOut,true);
@@ -280,6 +283,7 @@ namespace opkele {
 		(*(expat_t*)this) = parser_create_ns();
 		set_user_data(); set_element_handler();
 		set_character_data_handler();
+		set_unknown_encoding_handler();
 
 		if(xmode&xmode_html) {
 		    html_openid1.clear(); html_openid2.clear();
@@ -339,7 +343,7 @@ namespace opkele {
 		    const char *hv = ++colon;
 		    size_t hnl = colon-h;
 		    int rb;
-		    for(rb = bytes-hnl-1;rb>0 && isspace(*hv);++hv,--rb);
+		    for(rb = bytes-hnl-1;rb>0 && isspace(*hv);++hv,--rb) ;
 		    while(rb>0 && isspace(hv[rb-1])) --rb;
 		    if(rb) {
 			if( (hnl>=sizeof(XRDS_HEADER))
@@ -450,25 +454,40 @@ namespace opkele {
 		    --skipping; return;
 		}
 		if(is_qelement(n,NSURI_XRD "\tType")) {
-		    assert(xrd); assert(xrd_service); assert(cdata==&cdata_buf);
-		    xrd_service->types.insert(cdata_buf);
+		    if(xrd && xrd_service) {
+			assert(cdata==&cdata_buf);
+			xrd_service->types.insert(cdata_buf);
+		    }
 		}else if(is_qelement(n,NSURI_XRD "\tService")) {
-		    assert(xrd); assert(xrd_service);
-		    assert(!pt_stack.empty());
-		    assert(pt_stack.back()==(NSURI_XRD "\tService"));
-		    pt_stack.pop_back();
-		    xrd_service = 0;
-		}else if(is_qelement(n,NSURI_XRD "\tStatus")) {
-		    assert(xrd);
-		    if(is_qelement(pt_stack.back().c_str(),n)) {
-			assert(cdata==&status_string);
+		    if(!(xrd && xrd_service)) {
+			skipping = -1;
+		    }else{
+			assert(!pt_stack.empty());
+			assert(pt_stack.back()==(NSURI_XRD "\tService"));
 			pt_stack.pop_back();
-			if(status_code!=100)
-			    skipping = -1;
+			xrd_service = 0;
+		    }
+		}else if(is_qelement(n,NSURI_XRD "\tStatus")) {
+		    if(!xrd) {
+			skipping=-1;
+		    }else{
+			if(is_qelement(pt_stack.back().c_str(),n)) {
+			    assert(cdata==&status_string);
+			    pt_stack.pop_back();
+			    if(status_code!=100)
+				skipping = -1;
+			}
 		    }
 		}else if(is_qelement(n,NSURI_XRD "\tExpires")) {
-		    assert(xrd);
-		    xrd->expires = util::w3c_to_time(cdata_buf);
+		    if(!xrd) {
+			skipping=-1;
+		    }else{
+			xrd->expires = util::w3c_to_time(cdata_buf);
+		    }
+		}else if(is_qelement(n,NSURI_XRD "\tXRD")) {
+		    assert(!pt_stack.empty());
+		    assert(pt_stack.back()==(NSURI_XRD "\tXRD"));
+		    pt_stack.pop_back();
 		}else if((xmode&xmode_html) && is_element(n,"head")) {
 		    skipping = -1;
 		}
@@ -500,7 +519,7 @@ namespace opkele {
 			    rels.assign(a[1]);
 			}else if( !strcasecmp(a[0],"href") ) {
 			    const char *ns = a[1];
-			    for(;*ns && isspace(*ns);++ns);
+			    for(;*ns && isspace(*ns);++ns) ;
 			    href.assign(ns);
 			    string::size_type lns=href.find_last_not_of(data::_whitespace_chars);
 			    href.erase(lns+1);
@@ -566,6 +585,13 @@ namespace opkele {
 		}
 	    }
 
+	    int unknown_encoding(const XML_Char* /* n */,XML_Encoding *i) {
+		for(unsigned int ii=0;ii < sizeof(i->map)/sizeof(i->map[0]);++ii)
+		    i->map[ii] = ii;
+		i->convert = 0; i->release = 0;
+		return XML_STATUS_OK;
+	    }
+
     };
 
     string idiscover(endpoint_discovery_iterator oi,const string& identity) {
