@@ -16,7 +16,7 @@ void test_rfc_3986_normalize_uri(const string &ouri,bool success,const string& n
     try {
 	string n = opkele::util::rfc_3986_normalize_uri(ouri);
 	if(!success)
-	    throw failed_test(OPKELE_CP_ "Normalized URI when it shouldn't");
+	    throw failed_test(OPKELE_CP_ "Normalized URI when it shouldn't ('"+ouri+"' normalization resulted in '"+n+"')");
 	if(n!=nuri)
 	    throw failed_test(OPKELE_CP_ "rfc_3986_test_failed for '"+ouri+"' failed, expected '"+nuri+"', got '"+n+"'");
     }catch(opkele::bad_input& obi) {
@@ -31,8 +31,6 @@ void test_rfc_3986_normalize_uri(const string &ouri,bool success,const string& n
 void test_rfc_3986_normalize_uri() {
     test_rfc_3986_normalize_uri(
 	    "invalid", false );
-    test_rfc_3986_normalize_uri(
-	    "ftp://hacker.klever.net/", false );
     test_rfc_3986_normalize_uri(
 	    "http://", false );
     test_rfc_3986_normalize_uri(
@@ -75,10 +73,28 @@ void test_rfc_3986_normalize_uri() {
 	    "http://hacker.klever.net/ah/%5x", false );
     test_rfc_3986_normalize_uri(
 	    "Http://Hacker.Klever.Net:", true, "http://hacker.klever.net/" );
+    test_rfc_3986_normalize_uri(
+	    "http://www.xxx.com/openid/1", true, "http://www.xxx.com/openid/1" );
+}
+
+void test_w3c_to_time(const char *w3c,time_t expected) {
+    time_t t = opkele::util::w3c_to_time(w3c);
+    if(t!=expected) {
+	char tmp[512]; snprintf(tmp,sizeof(tmp)-1,
+		"test failed for %s, expected %lu, got %lu (expected-got == %lu)",
+		w3c, (unsigned long)expected, (unsigned long)t,
+		(unsigned long)(expected-t) );
+	throw failed_test(OPKELE_CP_ tmp);
+    }
+}
+
+void test_w3c_to_time() {
+    test_w3c_to_time("2008-06-29T12:33:44",1214742824);
 }
 
 int main() {
     try {
+	test_w3c_to_time();
 	test_rfc_3986_normalize_uri();
     }catch(failed_test& ft) {
 	cerr << "Test failed: " << ft.what() << endl;
